@@ -1,6 +1,6 @@
 /*
 spacenavd - a free software replacement driver for 6dof space-mice.
-Copyright (C) 2007-2009 John Tsiombikas <nuclear@member.fsf.org>
+Copyright (C) 2007-2010 John Tsiombikas <nuclear@member.fsf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,17 +19,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef CFGFILE_H_
 #define CFGFILE_H_
 
-#include "config.h"
+#include <limits.h>
 
 #define MAX_BUTTONS		64
 
 struct cfg {
-	float sensitivity;
+	float sensitivity, sens_trans, sens_rot;
 	int dead_threshold;
 	int invert[6];
 	int map_axis[6];
 	int map_button[MAX_BUTTONS];
 	int led;
+	char serial_dev[PATH_MAX];
 };
 
 void default_cfg(struct cfg *cfg);
